%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
%     function:   fnc_linkad_opt_c
%     
%     Continuous modulation order route optimization. Uses fmincon
%     
%     Input:
%     c_sys: system parameters
%     c_sim: simulation parameters
%     ch_gains: Link gain vector
%     
%     Output:
%     res: result vector of dimension Lx2, consists of
%         b: Modulation order, -1 if no valid config exists
%         P_tx: transmission power
%     
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function res = fnc_linkad_opt_c(c_sys, c_sim, ch_gains)
% Return transmission configuration: [b P_tx] (Both column vectors)
% b = -1 for unsuccessful configuration

%   b_ll and b_ul are vector of length L_route
%   lower and upper limit for b vector
    b_ll = c_sys.b_min*ones(1,c_sim.L_route);
    b_ul = fnc_linkad_find_b_max(c_sys, c_sim, ch_gains);
    
%     There is a link in the route too weak to be used. 
%     Return -1 for this and largest possible b for the others
    br_ind = find(b_ul < b_ll);    
    if (length(br_ind) > 0)
%         Debugging
%         disp('b_ul < b_ll in fnc_linkad_opt_c(...)')
%         b_ll
%         b_ul
        b_ul(br_ind) = -1;
        p_tx_vec = zeros(1,c_sim.L_route);
        res = [b_ul' p_tx_vec'];
        return
    end
    
%     Check if largest b meet time requirement
    T_min = sum(fnc_calc_T(c_sys, c_sim, b_ul));
    if(T_min > c_sim.T)
    
%         Debugging
%         disp('T_min > T_lim in fnc_linkad_opt_c(...)')
%         T_min
%         c_sim.T
        
        res = [-1*ones(c_sim.L_route,1) zeros(c_sim.L_route,1)];
        return
    
    end
    
%     Optimization routine
    options_tmp = optimset('fmincon');
%     options = optimset(options_tmp,'LargeScale','off', 'TolCon', 0, 'TolFun', 1e-6, 'TolX', 0.0001, 'Diagnostics', 'off', 'Display', 'off', 'FunValCheck', 'on');
    options = optimset(options_tmp,'LargeScale','off','TolCon', 1e-5, 'TolFun', 1e-3, 'TolX', 1e-3, 'Diagnostics', 'off', 'Display', 'off', 'FunValCheck', 'on');    
    [b_opt,exitflag] = fmincon(@E_tot, b_ul, [], [], [], [], b_ll, b_ul, @T_tot, options);
    % Extra output values, for debugging:
    % [b,fval,exitflag,output,lambda,grad,hessian] 
    % exitflag
    

    if (exitflag < 0)% No solution found, exitflag -1
        exitflag
        output
        lambda
%         pause
        res = [-1*ones(c_sim.L_route,1) zeros(c_sim.L_route,1)];
    else % Solution found
        p_tx_vec = fnc_calc_P_tx(c_sys, c_sim, b_opt, ch_gains);
        res = [b_opt' p_tx_vec'];
    end

 %%%%%%%% Nested functions   
    
%     Energy (Objective function)
    function E = E_tot(x)
%             Find transmission power associated with rate vector
            P_tx = fnc_calc_P_tx(c_sys, c_sim, x, ch_gains);
%             Find total energy consumption
            E = sum(fnc_calc_E(c_sys, c_sim, x, P_tx));            
   
    end

%     Time (Nonlinear constraint)
    function [c, ceq] = T_tot(y)
        
%         Time
        T_tot = sum(fnc_calc_T(c_sys, c_sim, y));
        c = T_tot - c_sim.T;
        ceq = [];
        
    end

end
