%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     function:   fnc_linkad_opt_d
%     
%     Discrete modulation order route optimization. Implements
%     energy-greedy optimization algorithm.
%     
%     Input:
%     c_sys: system parameters
%     c_sim: simulation parameters
%     ch_gains: Link gain vector
%     
%     Output:
%     res: result vector of dimension Lx2, consists of
%         b: Modulation order, -1 if no valid config exists
%         P_tx: transmission power
%     
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function res = fnc_linkad_opt_d(c_sys, c_sim, ch_gains)

res = zeros(c_sim.L_route,2);
E_chk = zeros(1,c_sim.L_route);
T_chk = zeros(1,c_sim.L_route);

% b_ul and b_ll are vectors of length L_route
b_ul = floor(fnc_linkad_find_b_max(c_sys, c_sim, ch_gains));
% b_ll = c_sys.b_min*ones(1,c_sim.L_route);

% Check if any links in the route are too weak
br_ind = find(b_ul < c_sys.b_min);
if (length(br_ind) > 0)    
    
%     Debugging:
%     disp('b_ul < b_ll in fnc_linkad_opt_d(...)')
%     b_ll
%     b_ul

    b_ul(br_ind) = -1;
    p_tx_vec = zeros(1,c_sim.L_route);
    
    res = [b_ul' p_tx_vec'];
    return
end

% Check if the largest possible b vector meets the time requirement
T_min = sum(fnc_calc_T(c_sys, c_sim, b_ul));
if (T_min > c_sim.T)
    
%     Debugging:
%     disp('T_min > T_lim in fnc_linkad_opt_d(...)')
%     T_min
%     c_sim.T

    res = [-1*ones(c_sim.L_route,1) zeros(c_sim.L_route,1)];
    return
    
end

% Initial check passed, running through algorithm
b_opt = b_ul;
p_tx_opt = fnc_calc_P_tx(c_sys, c_sim, b_opt, ch_gains);
E_opt = sum(fnc_calc_E(c_sys, c_sim, b_opt, p_tx_opt));

go = 1;
while (go)
    
    b_prev_opt = b_opt;
    
    for n = 1:c_sim.L_route
%         Decrease b with 1 for each hop
        b_tmp = b_prev_opt;
        b_tmp(n) = b_tmp(n) - 1;
        %Check if valid constellation size
        %Set invalid b to large value
        if (b_tmp(n) >= c_sys.b_min)
            T_tmp = sum(fnc_calc_T(c_sys, c_sim, b_tmp));
            if (T_tmp <= c_sim.T)
                p_tx_tmp = fnc_calc_P_tx(c_sys, c_sim, b_tmp, ch_gains);
                E_tmp = sum(fnc_calc_E(c_sys, c_sim, b_tmp, p_tx_tmp));
                if (E_tmp < E_opt)
                    E_opt = E_tmp;
                    b_opt = b_tmp; 
                    p_tx_opt = p_tx_tmp;
                end                
            end
        end
    end %end FOR

    if (b_prev_opt == b_opt)
        go = 0;
    end
    

end%End WHILE

    res = [b_opt' p_tx_opt'];

return