%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
%     function:   fnc_route_tx
%     
%     Packet transmission over given route, using given route
%     configuration. Hop-by-hop transmission, terminates if error in
%     previous hop.
%     
%     Input:
%     c_sys: system parameters
%     c_sim: simulation parameters
%     g_vec: Link gain vector
%     r_cnf: Route configuration (mod.order and tr.power)
%     fix_p: Boolen value. Transmit at max power if 1, use given congif if
%     0
%     
%     Output:
%     res_matr: Result matrix, dimestion Lx6, each row consists of 
%               per-hop results:
%         flag: status flag. 1: OK, 0, err in previous hop, -1: tr. error
%         b: modulation order
%         P: transmission power
%         T: transmission time
%         E: energy consumption
%         SNR: received SNR
%     
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function res_matr = fnc_route_tx(c_sys, c_sim, g_vec, r_cnf, fix_p)
% res_matr: [flag b P T E SNR]
% r_cnf: [b P]: -1 no valid b, -2 could not meet snr.

% TODO: make retransmission-routine!
% flag suggestion:
% -1: snr error
%  0: error in prev. hop
%  1: ok

% Initiate variables
res_matr = zeros(c_sim.L_route,6);
err_ind = find(r_cnf(:,1) < 1);

% Check if fixed power is to be used
if (fix_p)
   P_tx_max = fnc_calc_P_tx_max(c_sys, c_sim.b_fix);
   r_cnf(:,2) = ones(c_sim.L_route,1)*P_tx_max;
end

% Check if configuration is valid
if (length(err_ind) > 0)
    disp('Invalid configuration accepted in route_tx')
    r_cnf
    pause
end

% Transmit hop-by-hop. Cancel if a transmission error occurs
res_matr(1,:) = fnc_hop_tx(c_sys, c_sim, g_vec(1), r_cnf(1,1), r_cnf(1,2));
for(n = 2:c_sim.L_route)
    if(res_matr(n-1,1) == 1)
        res_matr(n,:) = fnc_hop_tx(c_sys, c_sim, g_vec(n), r_cnf(n,1), r_cnf(n,2));
    else
        res_matr(n,:) = [0 0 0 0 0 0];
    end
end

return