%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     function:   print_accum_res
%     
%     Calculates simulation averages and presents results.
%     
%     Input:
%     c_sys: system parameters
%     c_sim: simulation parameters
%     res: Result matrix, number of rows equals numbers of simulation runs
%     N_tot: Number of runs (for result validation)
%     
%     Output:
%     res: result vector of dimension Lx2, consists of
%         E_b_avg: Average energy consumption per bit
%         P_tx_avg: Average transmission power
%         err_rate: Total error rate
%     
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [E_b_avg, P_tx_avg, err_rate] = print_accum_res(c_sys, c_sim, res, N_tot)
% [flag E T b_avg P_avg] 1: OK, -1: setup, err, -2: Tx err

N_ok = length(find(res(:,1) == 1));
N_ok_re = length(find(res(:,1) == 2));
N_err_setup = length(find(res(:,1) == -1));
N_err_tx = length(find(res(:,1) == -2));

ind_ok = find(res(:,1) > 0);
res_ok = res(ind_ok,:);

if (N_ok + N_ok_re + N_err_setup + N_err_tx ~= N_tot)
  disp('Err/Corr count fault!')
  return
end

N_ok_tot = N_ok + N_ok_re;
N_err_tot = N_err_setup + N_err_tx;
err_rate = 100*(N_err_tot)/N_tot;

if (N_err_tot > 0)
    err_rate_su = 100*(N_err_setup/N_err_tot);
    err_rate_tx = 100*(N_err_tx/N_err_tot);
else
    err_rate_su = 0;
    err_rate_tx = 0;
end

% % % % Not implemented
% if (N_ok_re > 0)
%     retrans_rate = 100*(N_ok_re/N_ok_tot);
% else
%     retrans_rate = 0;
% end

if (N_ok_tot > 0)

    E_p_avg = mean(res_ok(:,2));
    E_p_max = max(res_ok(:,2));
    E_p_min = min(res_ok(:,2));

    E_b_avg = E_p_avg/c_sys.L_pkt;
    E_b_avg_dbm = 10*log10(E_b_avg);

    T_avg = mean(res_ok(:,3));
    T_max = max(res_ok(:,3));
    T_min = min(res_ok(:,3));

    b_avg = mean(res_ok(:,4));
    P_tx_avg = mean(res_ok(:,5));

    disp(['  *Energy per packet [mJ]:'])
    disp(['       - Mean: ' num2str(E_p_avg) ', Max: ' num2str(E_p_max) ', Min: ' num2str(E_p_min) ])
    disp(['  *Energy per bit [mJ]:'])
    disp(['       - Mean: ' num2str(E_b_avg) ' (' num2str(E_b_avg_dbm) ' dBm)'])
    disp(['  *Time per packet[ms]:'])
    disp(['       - Mean: ' num2str(T_avg) ', Max: ' num2str(T_max) ', Min: ' num2str(T_min) ])
    disp(['  *Bit per symbol (mean): ' num2str(b_avg) ])
    disp(['  *Tx power(mean) [mW]: ' num2str(P_tx_avg) ])
    disp(['  *Error rate: ' num2str(err_rate) ' % (Setup: ' num2str(err_rate_su) ', %, Tx: ' num2str(err_rate_tx) ' %)' ])
%     disp(['  *Retransmissions: ' num2str(retrans_rate) ' %'])
    
    if(T_max > c_sim.T)
        if (abs(T_max - c_sim.T) > 0.0001)
            disp('Time Capacity Breached!!!!')
            disp(['Time [s]: ' num2str( T_max/1000)])
            disp(['Limit [s]: ' num2str( c_sim.T/1000)])
            pause
        end
    end

else
    disp(['  All transmissions failed!'])
    disp(['  *Error rate: ' num2str(err_rate) ' % (Setup: ' num2str(err_rate_su) ', %, Tx: ' num2str(err_rate_tx) ' %)' ])
%     disp(['  *Retransmissions: ' num2str(retrans_rate) ' %'])
    E_b_avg = 1;
    P_tx_avg = 0;
end

return